% This is an example code for running the ScSPM algorithm described in "Linear 
% Spatial Pyramid Matching using Sparse Coding for Image Classification" (CVPR'09)
%
% Written by Jianchao Yang @ IFP UIUC
% For any questions, please email to jyang29@ifp.illinois.edu.
% 
% Revised May, 2010 by Jianchao Yang

clear all;
clc;

%% set path
addpath('sobel');
addpath(genpath('sparse_coding'));

%% parameter setting

% directory setup
img_dir = 'image';                  % directory for dataset images
data_dir = '4Dir_SP_from_txt_P16_F160_data';                  % directory to save the sift features of the chosen dataset
dataSet = 'Fonts';

% dictionary training for sparse coding
skip_dic_training = false;
nBases = 1024;
nsmp = 200000;

rt_img_dir = fullfile(img_dir, dataSet);
rt_data_dir = fullfile(data_dir, dataSet);

%% calculate sift features or retrieve the database directory
database = retr_database_dir(rt_data_dir);


%% load sparse coding dictionary (one dictionary trained on Caltech101 is provided: dict_Caltech101_1024.mat)
Bpath = ['dictionary/4Dir_SP_from_txt_F160_kmeans_dict_' dataSet '_' num2str(nBases) '.mat'];
Xpath = ['dictionary/4Dir_SP_from_txt_F160_Kmeans_rand_patches_' dataSet '_' num2str(nsmp) '.mat'];

if ~skip_dic_training,
    try 
        load(Xpath);
    catch
        X = rand_sampling(database, nsmp);
        save(Xpath, 'X');
    end
  %  [B, S, stat] = reg_sparse_coding(X, nBases, eye(nBases), beta, gamma, num_iters);
     [centers, assignments] = vl_kmeans(X,nBases,'Initialization', 'plusplus');
     save(Bpath,'centers');
%    save(Bpath, 'B', 'S', 'stat');
else
    load(Bpath);
    
end
%nBases = size(B, 2);                    % size of the dictionary