% =========================================================================
% An example code for the algorithm proposed in
%
%   
%   "Content-Independent Font Recognition on a Single Chinese Character using 
%    Sparse Representation".
%
%
% Written by Weikang Song @ ICST PKU
% Jan, 2015.
% =========================================================================

clear all; close all; clc;

% -------------------------------------------------------------------------
% parameter setting: all parameter are here
run('G:\Program Files\MATLAB\VLFEATROOT\Toolbox\vl_setup'); %set your own VLFEAT patch here.
pyramid = [1, 2 ,4];                % spatial block structure for the SPM
knn = 5;                            % number of neighbors for local coding
c = 10;                             % regularization parameter for linear SVM
                                    % in Liblinear package

nRounds = 5;                       % number of random test on the dataset
tr_num  = 250;                       % training examples per category
mem_block = 10000;                   % maxmum number of testing features loaded each time  

tmp_img_dir = 'image';       % parent directory for the image database                             
tmp_dir = '4Dir_SP_harris_F40_';       % parent directory for saving descriptors and features

dataSet = 'Fonts';
patchSize = 16;

skip_dic_training = false;
nBases = 1024;
nsmp = 100000;
% -------------------------------------------------------------------------
% generate path
img_dir = strcat(tmp_img_dir,'/',dataSet); % directory for the image database  
data_dir = strcat(tmp_dir,'data_','P_',num2str(patchSize),'/',dataSet); %directory for the local descriptor
fea_dir = strcat(tmp_dir,'feature_','P_',num2str(patchSize),'/',dataSet); %directory for features
% -------------------------------------------------------------------------
% set path
addpath('Liblinear/matlab');        % we use Liblinear package, you need 
                                    % download and compile the matlab codes
% -------------------------------------------------------------------------
% extract local descriptors;

extr_sobel_prewitt_4_direction_harris(img_dir, data_dir);

% -------------------------------------------------------------------------
% retrieve the directory of the database and load the codebook
database = retr_database_dir(data_dir);


if isempty(database),
    error('Data directory error!');
end

Bpath = strcat('dictionary/',tmp_dir,'kmeans_dict_','P_',num2str(patchSize),'_',num2str(nBases),'_',dataSet ,'.mat');
Xpath = strcat('dictionary/',tmp_dir,'Kmeans_rand_patches_','P_',num2str(patchSize),'_',num2str(nBases),'_',dataSet ,'.mat');

if ~skip_dic_training,
    try 
        load(Xpath);
    catch
        X = rand_sampling(database, nsmp);
        save(Xpath, 'X');
    end
  %  [B, S, stat] = reg_sparse_coding(X, nBases, eye(nBases), beta, gamma, num_iters);
     [centers, assignments] = vl_kmeans(X,nBases,'Initialization', 'plusplus');
     save(Bpath,'centers');
%    save(Bpath, 'B', 'S', 'stat');
else
    load(Bpath);
    
end

B = load(Bpath);
B = B.centers;
nCodebook = size(B, 2);              % size of the codebook

% -------------------------------------------------------------------------
% extract image features

dFea = sum(nCodebook*pyramid.^2);
nFea = length(database.path);

fdatabase = struct;
fdatabase.path = cell(nFea, 1);         % path for each image feature
fdatabase.label = zeros(nFea, 1);       % class label for each image feature

parfor iter1 = 1:nFea,  

    fpath = database.path{iter1};
    flabel = database.label(iter1);
    
    feature = load(fpath);
    [rtpath, fname] = fileparts(fpath);
    feaPath = fullfile(fea_dir, num2str(flabel), [fname '.mat']);
    if ~isdir(fullfile(fea_dir, num2str(flabel))),
        mkdir(fullfile(fea_dir, num2str(flabel)));
    end   

    par_fea = LLC_pooling(feature.feaSet, B, pyramid, knn);
    par_label = database.label(iter1);
  
    LLC_extr_par_save(feaPath, par_fea, par_label);
 end

for iter1 = 1:nFea,  

    fpath = database.path{iter1};
    flabel = database.label(iter1);

    [rtpath, fname] = fileparts(fpath);
    feaPath = fullfile(fea_dir, num2str(flabel), [fname '.mat']);
    if ~isdir(fullfile(fea_dir, num2str(flabel))),
        mkdir(fullfile(fea_dir, num2str(flabel)));
    end    
    fdatabase.label(iter1) = flabel;
    fdatabase.path{iter1} = feaPath;
end;

% -------------------------------------------------------------------------
% evaluate the performance of the image feature using linear SVM
% we used Liblinear package in this example code

fprintf('\n Testing...\n');
clabel = unique(fdatabase.label);
nclass = length(clabel);
accuracy = zeros(nRounds, 1);

for ii = 1:nRounds,
    fprintf('Round: %d...\n', ii);
    tr_idx = [];
    ts_idx = [];
    
    for jj = 1:nclass,
        idx_label = find(fdatabase.label == clabel(jj));
        num = length(idx_label);
        
        idx_rand = randperm(num);
        
        tr_idx = [tr_idx; idx_label(idx_rand(1:tr_num))];
        ts_idx = [ts_idx; idx_label(idx_rand(tr_num+1:end))];
    end
    
    fprintf('Training number: %d\n', length(tr_idx));
    fprintf('Testing number:%d\n', length(ts_idx));
    
    % load the training features 
    tr_fea = zeros(length(tr_idx), dFea);
    tr_label = zeros(length(tr_idx), 1);
    
    for jj = 1:length(tr_idx),
        fpath = fdatabase.path{tr_idx(jj)};
        load(fpath, 'fea', 'label');
        tr_fea(jj, :) = fea';
        tr_label(jj) = label;
    end
    
    options = ['-c ' num2str(c)];
    model = train(double(tr_label), sparse(tr_fea), options);
    clear tr_fea;

    % load the testing features
    ts_num = length(ts_idx);
    ts_label = [];
    
    if ts_num < mem_block,
        % load the testing features directly into memory for testing
        ts_fea = zeros(length(ts_idx), dFea);
        ts_label = zeros(length(ts_idx), 1);

        for jj = 1:length(ts_idx),
            fpath = fdatabase.path{ts_idx(jj)};
            load(fpath, 'fea', 'label');
            ts_fea(jj, :) = fea';
            ts_label(jj) = label;
        end

        [C] = predict(ts_label, sparse(ts_fea), model);
    else
        % load the testing features block by block
        num_block = floor(ts_num/mem_block);
        rem_fea = rem(ts_num, mem_block);
        
        curr_ts_fea = zeros(mem_block, dFea);
        curr_ts_label = zeros(mem_block, 1);
        
        C = [];
        
        for jj = 1:num_block,
            block_idx = (jj-1)*mem_block + (1:mem_block);
            curr_idx = ts_idx(block_idx); 
            
            % load the current block of features
            for kk = 1:mem_block,
                fpath = fdatabase.path{curr_idx(kk)};
                load(fpath, 'fea', 'label');
                curr_ts_fea(kk, :) = fea';
                curr_ts_label(kk) = label;
            end    
            
            % test the current block features
            ts_label = [ts_label; curr_ts_label];
            [curr_C] = predict(curr_ts_label, sparse(curr_ts_fea), model);
            C = [C; curr_C];
        end
        
        curr_ts_fea = zeros(rem_fea, dFea);
        curr_ts_label = zeros(rem_fea, 1);
        curr_idx = ts_idx(num_block*mem_block + (1:rem_fea));
        
        for kk = 1:rem_fea,
           fpath = fdatabase.path{curr_idx(kk)};
           load(fpath, 'fea', 'label');
           curr_ts_fea(kk, :) = fea';
           curr_ts_label(kk) = label;
        end  
        
        ts_label = [ts_label; curr_ts_label];
        [curr_C] = predict(curr_ts_label, sparse(curr_ts_fea), model); 
        C = [C; curr_C];        
    end
    
    % normalize the classification accuracy by averaging over different
    % classes
    acc = zeros(nclass, 1);

    for jj = 1 : nclass,
        c = clabel(jj);
        idx = find(ts_label == c);
        curr_pred_label = C(idx);
        curr_gnd_label = ts_label(idx);    
        acc(jj) = length(find(curr_pred_label == curr_gnd_label))/length(idx);
    end

    accuracy(ii) = mean(acc); 
    fprintf('Classification accuracy for round %d: %f\n', ii, accuracy(ii));
end

Ravg = mean(accuracy);                  % average recognition rate
Rstd = std(accuracy);                   % standard deviation of the recognition rate

fprintf('===============================================\n');
fprintf('Average classification accuracy: %f\n', Ravg);
fprintf('Standard deviation: %f\n', Rstd);    
fprintf('===============================================\n');
    
