% =========================================================================
% An example code to generate MSR
%
% Written by Weikang Song @ ICST PKU
% Jan, 2015.
% =========================================================================

clear all; close all; clc;

% -------------------------------------------------------------------------
% parameter setting: all parameter are here
run('G:\Program Files\MATLAB\VLFEATROOT\Toolbox\vl_setup'); %set your own VLFEAT patch here.
pyramid = [1, 2 ,4];                % spatial block structure for the SPM
knn = 5;                            % number of neighbors for local coding
c = 10;                             % regularization parameter for linear SVM
                                    % in Liblinear package 

tmp_img_dir = 'image';       % parent directory for the image database                             
tmp_dir = 'demo_4Dir_SP_rand_';       % parent directory for saving descriptors and features

dataSet = 'Demo_images';
patchSize = 16;
skip_dic_training = false;
nBases = 128;
nsmp = 1000;
% -------------------------------------------------------------------------
% generate path
img_dir = strcat(tmp_img_dir,'/',dataSet); % directory for the image database  
data_dir = strcat(tmp_dir,'data_','P_',num2str(patchSize),'/',dataSet); %directory for the local descriptor
fea_dir = strcat(tmp_dir,'feature_','P_',num2str(patchSize),'/',dataSet); %directory for features
% -------------------------------------------------------------------------
% set path
addpath('Liblinear/matlab');        % we use Liblinear package, you need 
                                    % download and compile the matlab codes
% -------------------------------------------------------------------------
% extract local descriptors;

 demo_extr_SPF_descriptor(img_dir, data_dir, patchSize);

% -------------------------------------------------------------------------
% retrieve the directory of the database and load the codebook
database = retr_database_dir(data_dir);


if isempty(database),
    error('Data directory error!');
end

Bpath = strcat('dictionary/',tmp_dir,'kmeans_dict_','P_',num2str(patchSize),'_',num2str(nBases),'_',dataSet ,'.mat');
Xpath = strcat('dictionary/',tmp_dir,'Kmeans_rand_patches_','P_',num2str(patchSize),'_',num2str(nBases),'_',dataSet ,'.mat');

if ~skip_dic_training,
    try 
        load(Xpath);
    catch
        X = rand_sampling(database, nsmp);
        save(Xpath, 'X');
    end
  %  [B, S, stat] = reg_sparse_coding(X, nBases, eye(nBases), beta, gamma, num_iters);
     [centers, assignments] = vl_kmeans(X,nBases,'Initialization', 'plusplus');
     save(Bpath,'centers');
%    save(Bpath, 'B', 'S', 'stat');
else
    load(Bpath);
    
end

B = load(Bpath);
B = B.centers;
nCodebook = size(B, 2);              % size of the codebook

% -------------------------------------------------------------------------
% extract image features

dFea = sum(nCodebook*pyramid.^2);
nFea = length(database.path);

fdatabase = struct;
fdatabase.path = cell(nFea, 1);         % path for each image feature
fdatabase.label = zeros(nFea, 1);       % class label for each image feature

parfor iter1 = 1:nFea,  

    fpath = database.path{iter1};
    flabel = database.label(iter1);
    
    feature = load(fpath);
    [rtpath, fname] = fileparts(fpath);
    feaPath = fullfile(fea_dir, num2str(flabel), [fname '.mat']);
    if ~isdir(fullfile(fea_dir, num2str(flabel))),
        mkdir(fullfile(fea_dir, num2str(flabel)));
    end   

    par_fea = LLC_pooling(feature.feaSet, B, pyramid, knn);
    par_label = database.label(iter1);
  
    LLC_extr_par_save(feaPath, par_fea, par_label);
 end